#pragma once

/****************************************************************************************
* Author:	Gilles Bellot
* Date:		11/07/2018 - Lenningen, Luxembourg
*
* Desc:		XAudio2
*
* History:
****************************************************************************************/

// INCLUDES /////////////////////////////////////////////////////////////////////////////

// C++ includes
#include <vector>

// Windows includes
#include <wrl.h>

// XAudio2 includes
#include <xaudio2.h>
#pragma comment(lib, "xaudio2.lib")

// Windows Media Foundation includes
#include <mfapi.h>

// bell0bytes util
#include "expected.h"

// CLASSES //////////////////////////////////////////////////////////////////////////////



namespace audio
{
	class AudioComponent;
	class SoundEvent;

	// the main audio engine powered by XAudio2 and Windows Media Foundation
	class AudioEngine
	{
	private:
		Microsoft::WRL::ComPtr<IXAudio2> dev;							// the main XAudio2 engine
		IXAudio2MasteringVoice* masterVoice;							// a mastering voice
		Microsoft::WRL::ComPtr<IMFAttributes> sourceReaderConfiguration;// Windows Media Foundation Source Reader Configuration

		util::Expected<void> initialize();								// this function initializes the XAudio2 interface
		util::Expected<void> loadFile(const std::wstring& filename, std::vector<BYTE>& audioData, WAVEFORMATEX** wafeFormatEx, unsigned int& waveLength);	// load audio file from disk

	public:
		// constructor and destructor
		AudioEngine();
		~AudioEngine();

		friend class AudioComponent;
	};
}