#pragma once

/****************************************************************************************
* Author:	Gilles Bellot
* Date:		11/07/2018 - Lenningen, Luxembourg
*
* Desc:		audio component
*
* History:
****************************************************************************************/

// INCLUDES /////////////////////////////////////////////////////////////////////////////

// C++ includes
#include <vector>

// Microsoft includes
#include <wrl.h>

// XAudio includes
#include <xaudio2.h>

// bell0bytes util
#include "expected.h"
#include "depesche.h"

// DEFINITIONS //////////////////////////////////////////////////////////////////////////

// CLASSES //////////////////////////////////////////////////////////////////////////////

namespace audio
{
	class AudioEngine;

	// sound event class to store data that can't be stored in RIFF files
	class SoundEvent
	{
	private:
		IXAudio2SourceVoice* sourceVoice;	// the XAudio2 source voice
		WAVEFORMATEX* waveFormat;			// the format of the audio file
		unsigned int waveLength;			// the length of the wave
		std::vector<BYTE> audioData;		// the audio data
		XAUDIO2_BUFFER audioBuffer;			// the actual buffer with the audio data

		float fallof;						// falloff distance
		unsigned int priority;				// music priority
		
		unsigned int index;					// the index of the actual sound to play
	public:
		SoundEvent();
		~SoundEvent();

		friend class AudioComponent;
	};

	class AudioComponent : public core::DepescheDestination
	{
	private:
		// the main audio engine
		AudioEngine* engine;				// the main audio engine: XAudio2 with Windows Media Component

		// handle message
		util::Expected<void> onMessage(const core::Depesche& depesche);
	public:
		// constructor and destructor
		AudioComponent();
		~AudioComponent();

		// load files from disk
		util::Expected<void> loadFile(const std::wstring fileName, SoundEvent& soundEvent);

		// play sound
		util::Expected<void> playSoundEvent(const SoundEvent& soundEvent);
		util::Expected<void> stopSoundEvent(const SoundEvent& soundEvent);
	};
}